/**
 * Defines Cell objects that are formed by a pair of numbers representing
 * the coordinates of a cell in the lattice
 * @author Damien Deville
 *
 */
public class Cell
{
	private int i ;				// The i-coordinate of the cell
	private int j ;				// The j-coordinate of the cell
	private boolean status ;	// If the cell is defined as occupied (true) or empty (false)
	private int clusterLabel ;	// The label of the cluster that the cell belongs to
	
	
	/**
	 * Constructor definition
	 * @param i: The i-coordinate of the cell
	 * @param j: The j-coordinate of the cell
	 * @param status: the status of the cell (true = occupied, false = empty)
	 */
	public Cell(int i, int j, boolean status)
	{
		setI(i) ;
		setJ(j) ;
		setStatus(status) ;
		setClusterLabel(-1) ;	// Initially, the cell belongs to no cluster
	}
	
	/**
	 * Constructor definition: if the user does not want to specify the status,
	 * we define the cell as empty
	 * @param i: The i-coordinate of the cell
	 * @param j: The j-coordinate of the cell
	 */
	public Cell(int i, int j)
	{
		setI(i) ;
		setJ(j) ;
		setStatus(false) ;
		setClusterLabel(-1) ;
	}
	
	/**
	 * Returns true if two coordinates are equal, otherwise returns false
	 * @param cell: the Cell object we want to compare it to
	 * @return true if the two objects are equal, otherwise false
	 */
	public boolean equalsTo(Cell cell)
	{
		if(i == cell.getI() && j == cell.getJ())
			return true ;
		else
			return false ;
	}
	
	/**
	 * Get the i-coordinate of the cell
	 * @return the i-coordinate of the cell
	 */
	public int getI()
	{
		return i ;
	}
	
	/**
	 * Get the j-coordinate of the cell
	 * @return the j-coordinate of the cell
	 */
	public int getJ()
	{
		return j ;
	}
	
	/**
	 * Get the status of the cell (occupied or empty)
	 * @return the status of the cell (occupied = true or empty = false)
	 */
	public boolean getStatus()
	{
		return status ;
	}
	
	/**
	 * Get the label of the cluster that the cell belongs to
	 * @return the label of the cluster that the cell belongs to
	 */
	public int getClusterLabel()
	{
		return clusterLabel ;
	}
	
	/**
	 * Set the i-coordinate of the cell
	 * @param i: the i-coordinate of the cell
	 */
	private void setI(int i)
	{
		this.i = i ;
	}
	
	/**
	 * Set the j-coordinate of the cell
	 * @param j: the j-coordinate of the cell
	 */
	private void setJ(int j)
	{
		this.j = j ;
	}
	
	/**
	 * Set the status of the cell (occupied or empty)
	 * @param status: the status of the cell (true = occupied, false = empty)
	 */
	public void setStatus(boolean status)
	{
		this.status = status ;
	}
	
	/**
	 * Set the label of the cluster that the cell belongs to
	 * @param clusterLabel: the label of the cluster that the cell belongs to
	 */
	public void setClusterLabel(int clusterLabel)
	{
		this.clusterLabel = clusterLabel ;
	}
}