import java.util.Enumeration;
import java.util.Hashtable;

/**
 * Defines functions used in order to get some results
 * from the model
 * @author Damien Deville
 *
 */
public class Functions
{
	
	/**
	 * Constructor definition 
	 */
	public Functions()
	{
		
	}
	
	/**
	 * Get the cluster size distribution for a given probability p
	 * print out frequency of each cluster size in the lattice
	 * @param p: the probability p
	 * @param latticeSize: the lattice size
	 * @param numExp: number of experiments
	 */
	public void clusterSizeDistribution(double p, int latticeSize, int numExp)
	{
		// HashTable that will contain the cluster size as a key and
		// the frequency as the corresponding value
		Hashtable<Integer,Double> nums = new Hashtable<Integer,Double>() ;	
		// we loop as many times as number of experiments
		for(int n = 0 ; n < numExp ; n++)
		{
			Lattice lat = new Lattice(latticeSize,p) ;	// create new lattice
			Clusters clusters = new Clusters(lat) ;	// create new clusters
			Integer[] sizes = clusters.getClusterSizes() ;	// get cluster sizes
			// loop as many times as the sizes of cluster sizes array
			for(int i = 0 ; i < sizes.length ; i++)
			{
				// if the cluster size already exists as a key in the hashtable
				if(nums.containsKey(sizes[i]))
					// we increment the value (frequency) by 1
					nums.put(sizes[i], nums.get(sizes[i])+1) ;
				else	// if it does not exist
					nums.put(sizes[i], 1.0) ;	// we create a new hashtable entry
			}
		}
		// we then print out the averaged frequency for each cluster size
		Enumeration<Integer> k = nums.keys() ;
		while (k.hasMoreElements())
		{
			int key = (int)k.nextElement() ;
			double value = (double)nums.get(key) ;
			System.out.println(key + " " + value/(double)numExp);
		}
	}
	
	
	/**
	 * Compute returns from the Cont-Bouchaud model given a probability a and
	 * summing over all clusters sizes for all probabilities p (step 0.01)
	 * between 0.01 and 0.59. Write down the returns in a text file
	 * @param a: the probability a
	 * @param latticeSize: the lattice size
	 * @param numSteps: number of time steps
	 * @param numExp: number of experiments
	 */
	public void returnsCBsumProb(double a, int latticeSize, int numSteps, int numExp)
	{
		String s = "output" ;
		String ext = ".txt" ;
		for(int j = 1 ; j <= numExp ; j++)	// loop as many times as number of experiments
		{
			String output = s + j + ext ;	// create txt file name
			System.out.println(output) ;
			Integer[] originalClusterSizes = new Integer[0] ;
			for(int n = 1 ; n <= 59 ; n++)	// loop through all probabilities p
			{
				double proba = (double)n/100 ;	// define the probability p
				Lattice lat = new Lattice(latticeSize, proba) ;	// create a new lattice
				Clusters clusters = new Clusters(lat) ;	// new clusters
				Integer[] clusterSizes = clusters.getClusterSizes() ;	// get all cluster sizes
				// create a new array containing previous and new cluster sizes
				Integer[] newClusterSizes = new Integer[originalClusterSizes.length + clusterSizes.length] ;
				// we add to it all previous clusters sizes
				for(int i = 0 ; i < originalClusterSizes.length ; i++)
					newClusterSizes[i] = originalClusterSizes[i] ;
				// we add to it all new clusters sizes
				for(int i = 0 ; i < clusterSizes.length ; i++)
					newClusterSizes[i+originalClusterSizes.length] = clusterSizes[i] ;
				originalClusterSizes = newClusterSizes ;
			}
			// given the new list of cluster sizes, we get the returns from the model
			Model model = new Model(numSteps, 1) ;
			double[] priceReturns = model.classicConstantContBouchaud(a, 1, originalClusterSizes) ;
			FileOutput out = new FileOutput(output) ;	// create txt file
			// for each time step, we write the result as a new line in the txt file
			for(int i = 0 ; i < priceReturns.length ; i++)
			{
				out.writeDouble(priceReturns[i]) ;
			    out.writeNewline() ;
			}
			out.close() ;	// close the text file
		}
	}
	
	
	/**
	 * Compute returns from the Cont-Bouchaud model given a probability a and
	 * a probability p. Write down the returns in a text file
	 * @param p: the probability p
	 * @param a: the probability a
	 * @param latticeSize: the lattice size
	 * @param numSteps: the number of time steps
	 * @param numExp: the number of experiments we average out
	 */
	public void returnsCBoneProb(double p, double a, int latticeSize, int numSteps, int numExp)
	{
		String s = "output" ;
		String ext = ".txt" ;	
		for(int n = 1 ; n <= numExp ; n++)	// loop as many times as number of experiments
		{
			String output = s + n + ext ;	// create txt file name
			System.out.println(output) ;
			FileOutput out = new FileOutput(output) ;	// create txt file
			Model model = new Model(numSteps, 1) ;	// create a new model object
			// we get the returns from the Cont-Bouchaud model
			double[] priceReturns = model.classicConstantContBouchaud(latticeSize, p, a, 1) ;			
			// for each time step, we write the result as a new line in the txt file
			for(int i = 0 ; i < numSteps ; i++)
			{
				out.writeDouble(priceReturns[i]) ;
			    out.writeNewline() ;
			}
			out.close() ;	// close the text file
		}
	}
	
	/**
	 * Compute prices from the Cont-Bouchaud model given a probability a and
	 * a probability p. Write down the returns in a text file
	 * @param p: the probability p
	 * @param a: the probability a
	 * @param latticeSize: lattice size
	 * @param numSteps: number of time steps
	 * @param numExp: number of experiments
	 * @param initialPrice: the initial asset price
	 * @param lambda: the price scaling factor
	 */
	public void timeSeriesCBoneProb(double p, double a, int latticeSize, int numSteps, int numExp, double initialPrice, double lambda)
	{
		String s = "output" ;
		String ext = ".txt" ;	
		for(int n = 1 ; n <= numExp ; n++)	// loop as many times as number of experiments
		{
			String output = s + n + ext ;	// create txt file name
			System.out.println(output) ;
			FileOutput out = new FileOutput(output) ;	// create txt file
			Model model = new Model(numSteps, initialPrice) ;	// create a new model object
			// we get the returns from the Cont-Bouchaud model
			double[] priceReturns = model.classicConstantContBouchaud(latticeSize, p, a, lambda) ;
			// we generate a time-series from the returns
			double[] timeSeries = model.generateTimeSeries(priceReturns) ;
			// for each time step, we write the result as a new line in the txt file
			for(int i = 0 ; i < numSteps ; i++)
			{
				out.writeDouble(timeSeries[i]) ;
			    out.writeNewline() ;
			}
			out.close() ;	// close the text file
		}
	}
	
	
	/**
	 * Find the critical probability checking if percolation occurs in each lattice
	 * for each p. Average the results out of a number of trials
	 * @param latticeSize: the lattice size
	 * @param trials: number of trials
	 */
	public void findCriticProb(int latticeSize, int trials)
	{
		Lattice lat ;	// define new lattice
		double p ;		// probability p
		int probMesh = 100 ;	// the probability mesh size
		//long seed = 12318991921L ; 
		int[] results = new int[probMesh] ;	// array containing results
		int count ;
		for(int i = 1 ; i <= probMesh ; i++)
		{
			p = (double)i/probMesh ;	// defines the probability p
			count = 0 ;
			// loop as many times as trials number
			for(int j = 0 ; j < trials ; j++)
			{
				System.out.println("Trial number: " + j) ;
				lat = new Lattice(latticeSize, p) ;	// create a new lattice
				Clusters clusters = new Clusters(lat) ;	// get the clusters
				if(clusters.checkPercolation())	// check if percolation occurs
					count++ ;	// if percolation occurs, increase count by 1
			}
			results[i-1] = count ;
			System.out.println(p + " " + results[i-1]) ;
		}
	}
	
	
	/**
	 * Print the average cluster number for each probability p
	 * @param latticeSize: the lattice size
	 * @param numExp: number of experiments to average out
	 */
	public void averageClusterNum(int latticeSize, int numExp)
	{
		for(double i = 1 ; i <= 100 ; i++)
		{
			double prob = i / 100 ;	// probability p
			int sum = 0 ;
			for(int j = 0 ; j < numExp ; j++)
			{
				//long seed = 123189914592L ;
				Lattice lat = new Lattice(latticeSize, prob) ;
				Clusters clusters = new Clusters(lat) ;
				sum += clusters.getClusterNumber() ;
			}
			System.out.println(prob + " " + (double)sum/numExp) ;
		}
	}	
}